var _ = require('underscore');
var Link = require('../../models/link');
var Account = require('../../models/account');
var readLink = require('../../read-link');
var authHelper = require('../auth/helpers');

var getLinks = function(user_id, cb) {
  Link.find()
  .where('_creator').equals(user_id)
  .sort('-addedDate')
  .exec(cb);
}

var renderErr = function(res, err, message) {
  res.status(400);
  res.render('error', {
    message: message,
    error: err
  });
}

var updateLink = function(link, summary, cb) {
    link.title = summary.title;
    link.image = summary.image;
    link.sentiment = summary.sentiment;
    link.keywords = summary.topics;
    link.words = summary.words;
    link.difficulty = summary.difficulty;
    link.readingTime = summary.minutes;
    link.save(cb);
}

var getRoot = function(req, res) {
  res.render('links', _.extend(res.ctx, {
    title: 'Your links',
    linkid: null
  }));
}

var redirectAfterNewLink = function(err, link, count) {
  if(err) 
    return renderErr(res, err, 'Could not save link');

  res.redirect('/links/'+link._id);
}

var createNewLink = function(err, summary) {
  if(err) 
    return renderErr(res, err, 'Failed to fetch link: '+req.param('link'));

  var link = new Link({
    url: req.param('link'),
    _creator: req.user._id
  });

  updateLink(link, summary, redirectAfterNewLink)
}

var postRoot = function(req, res) {

  readLink(req.param('link'), createNewLink);
}

var linkMiddleware = function(req, res, next) {
  res.ctx.linkid = req.params.linkid;

  Link.findOne({
    '_creator': req.user._id, 
    '_id': res.ctx.linkid
  }, function(err, link, count) {
    if(err || !link) 
      return renderErr(res, err, 'Not found');

    res.ctx.link = link;
    next();
  })
}

var linksMiddleware = function(req, res, next) {
  if(!req.user)
    res.redirect('/auth/twitter');

  res.ctx = {user: req.user};
  getLinks(req.user._id, function(err, links, count) {
    if(err) 
      return renderErr(res, err, 'Could not load links');

    res.ctx.links = links
    next();
  })
}

var getLink = function(req, res) {
  res.render('links', _.extend(res.ctx, {
    title: 'Your links',
  }))
}

var putLink = function(req, res) {
  readLink(res.ctx.link.url, function(err, summary) {
    if(err) 
      return renderErr(res, err, 'Failed to fetch link: '+req.param('link'));

    res.ctx.link.addedDate = Date.now();

    updateLink(res.ctx.link, summary, function(err, link, count) {
      if(err) 
        return renderErr(res, err, 'Could not save link');

      res.send('OK');
    });
  })
}

var removeLink = function(req, res) {
  res.ctx.link.remove(function(err, link) {
    res.send('Remove successful');
  })
}

module.exports = {
  getLinks: getLinks,
  getRoot: getRoot,
  postRoot: postRoot,
  linksMiddleware: linksMiddleware,
  linkMiddleware: linkMiddleware,
  getLink: getLink,
  putLink: putLink,
  removeLink: removeLink
}